from aiwolf_nlp_common.client import Client


def test_split_receive_info() -> None:
    log_name_json: str = """{"request": "NAME"}"""
    multi_json: str = (
        """{"request": "NAME"}\n{"request": "NAME"}\n{"request": "NAME"}\n{"request": "NAME"}\n{"request": "NAME"}\n"""
    )

    correct_list: list = [log_name_json] * 5

    assert Client.split_receives(receive=multi_json) == correct_list

    multi_json: str = """{"request": "NAME"}\n"""
    assert Client.split_receives(receive=multi_json) == [log_name_json]



from __future__ import annotations

from aiwolf_nlp_common.role import RoleInfo


class ExistingRoleList(list):
    @classmethod
    def initialize_from_json(cls, value: list[str] | None = None) -> ExistingRoleList:
        if value is None:
            return cls()
        return cls([RoleInfo.get_role_info(role=role) for role in value])



from __future__ import annotations

import enum


class Action(enum.Enum):
    INITIALIZE = "INITIALIZE"
    NAME = "NAME"
    ROLE = "ROLE"
    DAILY_INITIALIZE = "DAILY_INITIALIZE"
    DAILY_FINISH = "DAILY_FINISH"
    TALK = "TALK"
    VOTE = "VOTE"
    DIVINE = "DIVINE"
    ATTACK = "ATTACK"
    WHISPER = "WHISPER"
    FINISH = "FINISH"

    @classmethod
    def is_initialize(cls, request: str | Action) -> bool:
        return request in {cls.INITIALIZE, cls.INITIALIZE.value}

    @classmethod
    def is_name(cls, request: str | Action) -> bool:
        return request in {cls.NAME, cls.NAME.value}

    @classmethod
    def is_role(cls, request: str | Action) -> bool:
        return request in {cls.ROLE, cls.ROLE.value}

    @classmethod
    def is_daily_initialize(cls, request: str | Action) -> bool:
        return request in {cls.DAILY_INITIALIZE, cls.DAILY_INITIALIZE.value}

    @classmethod
    def is_daily_finish(cls, request: str | Action) -> bool:
        return request in {cls.DAILY_FINISH, cls.DAILY_FINISH.value}

    @classmethod
    def is_talk(cls, request: str | Action) -> bool:
        return request in {cls.TALK, cls.TALK.value}

    @classmethod
    def is_vote(cls, request: str | Action) -> bool:
        return request in {cls.VOTE, cls.VOTE.value}

    @classmethod
    def is_divine(cls, request: str | Action) -> bool:
        return request in {cls.DIVINE, cls.DIVINE.value}

    @classmethod
    def is_attack(cls, request: str | Action) -> bool:
        return request in {cls.ATTACK, cls.ATTACK.value}

    @classmethod
    def is_whisper(cls, request: str | Action) -> bool:
        return request in {cls.WHISPER, cls.WHISPER.value}

    @classmethod
    def is_finish(cls, request: str | Action) -> bool:
        return request in {cls.FINISH, cls.FINISH.value}



from aiwolf_nlp_common.protocol.communication_protocol import CommunicationProtocol
from aiwolf_nlp_common.protocol.gameSetting.map.role_num_map import (
    RoleNumInfo,
    RoleNumMap,
)


def test_request(
    name_str: str, name_json: dict, initialize_str: str, initialize_json: dict
):
    test_protocol = CommunicationProtocol.initialize_from_json(value=name_str)

    assert test_protocol.request == name_json["request"]

    test_protocol = CommunicationProtocol.initialize_from_json(value=initialize_str)

    assert test_protocol.request == initialize_json["request"]


def test_game_info(initialize_str: str, initialize_json: dict) -> None:
    test_protocol = CommunicationProtocol.initialize_from_json(value=initialize_str)

    assert test_protocol.game_info.day == initialize_json["gameInfo"]["day"]
    assert test_protocol.game_info.agent == initialize_json["gameInfo"]["agent"]


def test_game_setting(initialize_str: str, initialize_json: dict) -> None:
    result = CommunicationProtocol.initialize_from_json(value=initialize_str)

    check_role_num_map = RoleNumMap()

    for role in initialize_json["gameSetting"]["roleNumMap"].keys():
        add_elem = RoleNumInfo(
            role=role,
            allocated_count=initialize_json["gameSetting"]["roleNumMap"][role],
        )
        check_role_num_map.add(add_elem)

    assert result.game_setting.role_num_map == check_role_num_map
    assert result.game_setting.max_talk == initialize_json["gameSetting"]["maxTalk"]
    assert (
        result.game_setting.action_timeout
        == initialize_json["gameSetting"]["actionTimeout"]
    )



from __future__ import annotations

from dataclasses import dataclass


@dataclass(frozen=True)
class TalkInfo:
    agent: str
    day: int
    idx: int
    text: str
    turn: str
    skip: bool
    over: bool

    def __str__(self) -> str:
        return f"index:{self.idx}, {self.agent}: {self.text}"

    def is_skip(self) -> bool:
        return self.skip

    def is_over(self) -> bool:
        return self.over

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, TalkInfo):
            return False
        return (
            self.agent == other.agent
            and self.day == other.day
            and self.idx == other.idx
            and self.text == other.text
            and self.turn == other.turn
            and self.skip == other.skip
            and self.over == other.over
        )


class TalkList(list[TalkInfo]):
    def __str__(self) -> str:
        header = f"[{self.__class__.__name__}]"
        return (
            f"{header}\nNo Result Available"
            if self.is_empty()
            else f"{header}" + "\n".join(str(elem) for elem in self)
        )

    @classmethod
    def initialize_from_json(cls, value: list[dict] | None) -> TalkList:
        instance = cls()
        if not value:
            return instance
        instance.extend(
            TalkInfo(
                agent=talk_info["agent"],
                day=talk_info["day"],
                idx=talk_info["idx"],
                text=talk_info["text"],
                turn=talk_info["turn"],
                skip=talk_info["skip"],
                over=talk_info["over"],
            )
            for talk_info in value
        )
        return instance

    def __init__(self, value: list[dict] | None = None) -> None:
        self.initialize_from_json(value)

    def is_empty(self) -> bool:
        return not bool(self)



import json
import re

import pytest

from aiwolf_nlp_common import util

# from AIWolfNLAgentPython.player.agent import Agent
# from AIWolfNLAgentPython.player.possessed import Possessed
# from AIWolfNLAgentPython.player.seer import Seer
# from AIWolfNLAgentPython.player.villager import Villager
# from AIWolfNLAgentPython.player.werewolf import Werewolf


@pytest.fixture
def name_str() -> str:
    return """{"request": "NAME"}"""


@pytest.fixture
def name_json(name_str) -> dict:
    return json.loads(name_str)


@pytest.fixture
def initialize_str() -> str:
    return """{"request":"INITIALIZE","info":{"statusMap":{"Agent[01]":"ALIVE","Agent[02]":"ALIVE","Agent[03]":"ALIVE","Agent[04]":"ALIVE","Agent[05]":"ALIVE"},"roleMap":{"Agent[01]":"WEREWOLF"},"remainTalkMap":{},"remainWhisperMap":{},"day":0,"agent":"Agent[01]"},"setting":{"roleNumMap":{"BODYGUARD":0,"MEDIUM":0,"POSSESSED":1,"SEER":1,"VILLAGER":2,"WEREWOLF":1},"playerNum":5,"maxTalk":3,"maxTalkTurn":15,"maxWhisper":3,"maxWhisperTurn":15,"maxSkip":3,"isEnableNoAttack":true,"isVoteVisible":true,"isTalkOnFirstDay":true,"responseTimeout":90000,"actionTimeout":60000,"maxRevote":1,"maxAttackRevote":1}}"""


@pytest.fixture
def initialize_json(initialize_str) -> dict:
    return json.loads(initialize_str)


@pytest.fixture
def daily_initialize_str() -> str:
    return """{"request":"DAILY_INITIALIZE","gameInfo":{"day":0,"agent":"Agent[01]","voteList":[],"latestVoteList":[],"attackVoteList":[],"latestAttackVoteList":[],"talkList":[],"whisperList":[],"statusMap":{"Agent[01]":"ALIVE","Agent[02]":"ALIVE","Agent[03]":"ALIVE","Agent[04]":"ALIVE","Agent[05]":"ALIVE"},"roleMap":{"Agent[01]":"VILLAGER"},"remainTalkMap":{"Agent[01]":5,"Agent[02]":5,"Agent[03]":5,"Agent[04]":5,"Agent[05]":5},"remainWhisperMap":{"Agent[04]":5},"existingRoleList":["POSSESSED","SEER","VILLAGER","WEREWOLF"],"lastDeadAgentList":[]},"gameSetting":{"roleNumMap":{"ANY":0,"FREEMASON":0,"FOX":0,"POSSESSED":1,"BODYGUARD":0,"MEDIUM":0,"VILLAGER":2,"WEREWOLF":1,"SEER":1},"maxTalk":5,"maxTalkTurn":20,"maxWhisper":5,"maxWhisperTurn":20,"maxSkip":0,"isEnableNoAttack":false,"isVoteVisible":false,"isTalkOnFirstDay":true,"responseTimeout":6000,"actionTimeout":3000,"maxRevote":0,"maxAttackRevote":0,"isEnableRoleRequest":false,"playerNum":5}}"""


@pytest.fixture
def daily_initialize_json(daily_initialize_str) -> dict:
    return json.loads(daily_initialize_str)


@pytest.fixture
def talk_str() -> str:
    return """{"request":"TALK","talkHistory":[{"idx":0,"day":0,"turn":0,"agent":"Agent[03]","text":">>Agent[04] Agent[04]は他の運び屋と違い、占い師に人狼判定を出されていて、人狼判定を占いに出していないと報告されています！","skip":false,"over":false},{"idx":1,"day":0,"turn":0,"agent":"Agent[01]","text":">>Agent[05] そう。ボクがこの村の占い師なのさ。","skip":false,"over":false}],"whisperHistory":[]}"""


@pytest.fixture
def talk_json(talk_str) -> dict:
    return json.loads(talk_str)


@pytest.fixture
def daily_finish_str() -> str:
    return """{"request":"DAILY_FINISH","talkHistory":[{"idx":24,"day":0,"turn":4,"agent":"Agent[04]","text":"安心してください！　Agent[05]が人狼か？と聞かれてもYESとはいいません！　イエス、というだけです！データバンクにアクセスしてみた結果占いCOしていないと報告されています！","skip":false,"over":false},{"idx":25,"day":0,"turn":5,"agent":"Agent[05]","text":"Over","skip":false,"over":true},{"idx":26,"day":0,"turn":5,"agent":"Agent[04]","text":"Over","skip":false,"over":true},{"idx":27,"day":0,"turn":5,"agent":"Agent[03]","text":"Over","skip":false,"over":true},{"idx":28,"day":0,"turn":5,"agent":"Agent[01]","text":"Over","skip":false,"over":true},{"idx":29,"day":0,"turn":5,"agent":"Agent[02]","text":"Over","skip":false,"over":true}],"whisperHistory":[]}"""


@pytest.fixture
def daily_finish_json(daily_finish_str) -> dict:
    return json.loads(daily_finish_str)


@pytest.fixture
def divine_str() -> str:
    return """{"request":"DIVINE","talkHistory":[],"whisperHistory":[]}"""


@pytest.fixture
def divine_json(divine_str) -> dict:
    return json.loads(divine_str)


@pytest.fixture
def vote_str() -> str:
    return """{"request":"VOTE","talkHistory":[],"whisperHistory":[]}"""


@pytest.fixture
def vote_json(vote_str) -> dict:
    return json.loads(vote_str)


@pytest.fixture
def attack_str() -> str:
    return """{"request":"ATTACK","gameInfo":{"day":2,"agent":"Agent[04]","executedAgent":"Agent[01]","latestExecutedAgent":"Agent[03]","attackedAgent":"Agent[02]","voteList":[],"latestVoteList":[],"attackVoteList":[{"day":1,"agent":"Agent[04]","target":"Agent[02]"}],"latestAttackVoteList":[],"talkList":[{"idx":17,"day":2,"turn":5,"agent":"Agent[04]","text":"Over","skip":false,"over":true}],"whisperList":[],"statusMap":{"Agent[01]":"DEAD","Agent[02]":"DEAD","Agent[03]":"DEAD","Agent[04]":"ALIVE","Agent[05]":"ALIVE"},"roleMap":{"Agent[04]":"WEREWOLF"},"remainTalkMap":{"Agent[03]":0,"Agent[04]":0,"Agent[05]":5},"remainWhisperMap":{"Agent[04]":5},"existingRoleList":["POSSESSED","SEER","VILLAGER","WEREWOLF"],"lastDeadAgentList":["Agent[02]"]}}"""


@pytest.fixture
def attack_json(attack_str) -> dict:
    return json.loads(attack_str)


@pytest.fixture
def finish_str() -> str:
    return """{"request":"FINISH","gameInfo":{"day":3,"agent":"Agent[01]","executedAgent":"Agent[03]","voteList":[],"latestVoteList":[],"attackVoteList":[],"latestAttackVoteList":[],"talkList":[],"whisperList":[],"statusMap":{"Agent[01]":"DEAD","Agent[02]":"DEAD","Agent[03]":"DEAD","Agent[04]":"ALIVE","Agent[05]":"DEAD"},"roleMap":{"Agent[01]":"VILLAGER","Agent[02]":"POSSESSED","Agent[03]":"VILLAGER","Agent[04]":"WEREWOLF","Agent[05]":"SEER"},"remainTalkMap":{"Agent[04]":5},"remainWhisperMap":{"Agent[04]":5},"existingRoleList":["POSSESSED","SEER","VILLAGER","WEREWOLF"],"lastDeadAgentList":["Agent[05]"]}}"""


@pytest.fixture
def finish_json(finish_str) -> dict:
    return json.loads(finish_str)


def json_fix(json_str: str) -> str:
    json_str = json_str.replace("'", '"')
    json_str = re.sub('[a-zA-Z]"[a-zA-Z]', "'", json_str)
    json_str = json_str.replace("False", '"False"')
    json_str = json_str.replace("True", '"True"')

    return json_str


@pytest.fixture
def role_num_map() -> dict:
    role_json = """{'request': 'INITIALIZE', 'gameInfo': {'day': 0, 'agent': 'Agent[02]', 'voteList': [], 'latestVoteList': [], 'attackVoteList': [], 'latestAttackVoteList': [], 'talkList': [], 'whisperList': [], 'statusMap': {'Agent[01]': 'ALIVE', 'Agent[02]': 'ALIVE', 'Agent[03]': 'ALIVE', 'Agent[04]': 'ALIVE', 'Agent[05]': 'ALIVE'}, 'roleMap': {'Agent[02]': 'VILLAGER'}, 'remainTalkMap': {'Agent[01]': 5, 'Agent[02]': 5, 'Agent[03]': 5, 'Agent[04]': 5, 'Agent[05]': 5}, 'remainWhisperMap': {'Agent[05]': 5}, 'existingRoleList': ['POSSESSED', 'SEER', 'VILLAGER', 'WEREWOLF'], 'lastDeadAgentList': []}, 'gameSetting': {'roleNumMap': {'SEER': 1, 'FOX': 0, 'ANY': 0, 'WEREWOLF': 1, 'MEDIUM': 0, 'POSSESSED': 1, 'VILLAGER': 2, 'BODYGUARD': 0, 'FREEMASON': 0}, 'maxTalk': 5, 'maxTalkTurn': 20, 'maxWhisper': 5, 'maxWhisperTurn': 20, 'maxSkip': 0, 'isEnableNoAttack': False, 'isVoteVisible': False, 'isTalkOnFirstDay': True, 'responseTimeout': 120000, 'actionTimeout': 1200000, 'maxRevote': 0, 'maxAttackRevote': 0, 'isEnableRoleRequest': False, 'playerNum': 5}}\n"""
    role_json = json_fix(json_str=role_json)
    role_json = json.loads(role_json)

    return role_json["gameSetting"]["roleNumMap"]


@pytest.fixture
def seer_text() -> str:
    text = """{'request': 'INITIALIZE', 'gameInfo': {'day': 0, 'agent': 'Agent[03]', 'voteList': [], 'latestVoteList': [], 'attackVoteList': [], 'latestAttackVoteList': [], 'talkList': [], 'whisperList': [], 'statusMap': {'Agent[01]': 'ALIVE', 'Agent[02]': 'ALIVE', 'Agent[03]': 'ALIVE', 'Agent[04]': 'ALIVE', 'Agent[05]': 'ALIVE'}, 'roleMap': {'Agent[03]': 'SEER'}, 'remainTalkMap': {'Agent[01]': 5, 'Agent[02]': 5, 'Agent[03]': 5, 'Agent[04]': 5, 'Agent[05]': 5}, 'remainWhisperMap': {'Agent[05]': 5}, 'existingRoleList': ['POSSESSED', 'SEER', 'VILLAGER', 'WEREWOLF'], 'lastDeadAgentList': []}, 'gameSetting': {'roleNumMap': {'SEER': 1, 'FOX': 0, 'ANY': 0, 'WEREWOLF': 1, 'MEDIUM': 0, 'POSSESSED': 1, 'VILLAGER': 2, 'BODYGUARD': 0, 'FREEMASON': 0}, 'maxTalk': 5, 'maxTalkTurn': 20, 'maxWhisper': 5, 'maxWhisperTurn': 20, 'maxSkip': 0, 'isEnableNoAttack': False, 'isVoteVisible': False, 'isTalkOnFirstDay': True, 'responseTimeout': 120000, 'actionTimeout': 1200000, 'maxRevote': 0, 'maxAttackRevote': 0, 'isEnableRoleRequest': False, 'playerNum': 5}}\n"""
    return json_fix(json_str=text)


@pytest.fixture
def seer_json(seer_text) -> dict:
    return json.loads(seer_text)


@pytest.fixture
def villager_text() -> str:
    text = """{'request': 'INITIALIZE', 'gameInfo': {'day': 0, 'agent': 'Agent[02]', 'voteList': [], 'latestVoteList': [], 'attackVoteList': [], 'latestAttackVoteList': [], 'talkList': [], 'whisperList': [], 'statusMap': {'Agent[01]': 'ALIVE', 'Agent[02]': 'ALIVE', 'Agent[03]': 'ALIVE', 'Agent[04]': 'ALIVE', 'Agent[05]': 'ALIVE'}, 'roleMap': {'Agent[02]': 'VILLAGER'}, 'remainTalkMap': {'Agent[01]': 5, 'Agent[02]': 5, 'Agent[03]': 5, 'Agent[04]': 5, 'Agent[05]': 5}, 'remainWhisperMap': {'Agent[05]': 5}, 'existingRoleList': ['POSSESSED', 'SEER', 'VILLAGER', 'WEREWOLF'], 'lastDeadAgentList': []}, 'gameSetting': {'roleNumMap': {'SEER': 1, 'FOX': 0, 'ANY': 0, 'WEREWOLF': 1, 'MEDIUM': 0, 'POSSESSED': 1, 'VILLAGER': 2, 'BODYGUARD': 0, 'FREEMASON': 0}, 'maxTalk': 5, 'maxTalkTurn': 20, 'maxWhisper': 5, 'maxWhisperTurn': 20, 'maxSkip': 0, 'isEnableNoAttack': False, 'isVoteVisible': False, 'isTalkOnFirstDay': True, 'responseTimeout': 120000, 'actionTimeout': 1200000, 'maxRevote': 0, 'maxAttackRevote': 0, 'isEnableRoleRequest': False, 'playerNum': 5}}\n"""

    return json_fix(json_str=text)


@pytest.fixture
def villager_json(villager_text) -> dict:
    return json.loads(villager_text)


@pytest.fixture
def possessed_text() -> str:
    text = """{'request': 'INITIALIZE', 'gameInfo': {'day': 0, 'agent': 'Agent[01]', 'voteList': [], 'latestVoteList': [], 'attackVoteList': [], 'latestAttackVoteList': [], 'talkList': [], 'whisperList': [], 'statusMap': {'Agent[01]': 'ALIVE', 'Agent[02]': 'ALIVE', 'Agent[03]': 'ALIVE', 'Agent[04]': 'ALIVE', 'Agent[05]': 'ALIVE'}, 'roleMap': {'Agent[01]': 'POSSESSED'}, 'remainTalkMap': {'Agent[01]': 5, 'Agent[02]': 5, 'Agent[03]': 5, 'Agent[04]': 5, 'Agent[05]': 5}, 'remainWhisperMap': {'Agent[05]': 5}, 'existingRoleList': ['POSSESSED', 'SEER', 'VILLAGER', 'WEREWOLF'], 'lastDeadAgentList': []}, 'gameSetting': {'roleNumMap': {'SEER': 1, 'FOX': 0, 'ANY': 0, 'WEREWOLF': 1, 'MEDIUM': 0, 'POSSESSED': 1, 'VILLAGER': 2, 'BODYGUARD': 0, 'FREEMASON': 0}, 'maxTalk': 5, 'maxTalkTurn': 20, 'maxWhisper': 5, 'maxWhisperTurn': 20, 'maxSkip': 0, 'isEnableNoAttack': False, 'isVoteVisible': False, 'isTalkOnFirstDay': True, 'responseTimeout': 120000, 'actionTimeout': 1200000, 'maxRevote': 0, 'maxAttackRevote': 0, 'isEnableRoleRequest': False, 'playerNum': 5}}\n"""
    return json_fix(json_str=text)


@pytest.fixture
def possessed_json(possessed_text) -> dict:
    return json.loads(possessed_text)


@pytest.fixture
def werewolf_text() -> str:
    text = """{'request': 'INITIALIZE', 'gameInfo': {'day': 0, 'agent': 'Agent[02]', 'voteList': [], 'latestVoteList': [], 'attackVoteList': [], 'latestAttackVoteList': [], 'talkList': [], 'whisperList': [], 'statusMap': {'Agent[01]': 'ALIVE', 'Agent[02]': 'ALIVE', 'Agent[03]': 'ALIVE', 'Agent[04]': 'ALIVE', 'Agent[05]': 'ALIVE'}, 'roleMap': {'Agent[02]': 'WEREWOLF'}, 'remainTalkMap': {'Agent[01]': 5, 'Agent[02]': 5, 'Agent[03]': 5, 'Agent[04]': 5, 'Agent[05]': 5}, 'remainWhisperMap': {'Agent[02]': 5}, 'existingRoleList': ['POSSESSED', 'SEER', 'VILLAGER', 'WEREWOLF'], 'lastDeadAgentList': []}, 'gameSetting': {'roleNumMap': {'BODYGUARD': 0, 'VILLAGER': 2, 'FOX': 0, 'SEER': 1, 'WEREWOLF': 1, 'FREEMASON': 0, 'MEDIUM': 0, 'POSSESSED': 1, 'ANY': 0}, 'maxTalk': 5, 'maxTalkTurn': 20, 'maxWhisper': 5, 'maxWhisperTurn': 20, 'maxSkip': 0, 'isEnableNoAttack': False, 'isVoteVisible': False, 'isTalkOnFirstDay': True, 'responseTimeout': 120000, 'actionTimeout': 1200000, 'maxRevote': 0, 'maxAttackRevote': 0, 'isEnableRoleRequest': False, 'playerNum': 5}}\n"""
    return json_fix(json_str=text)


@pytest.fixture
def werewolf_json(werewolf_text) -> dict:
    return json.loads(werewolf_text)


@pytest.fixture
def text_file_path() -> str:
    return "tests/AIWolfNLAgentPython/res/2019071_44011_AIWolfTalkLogs.txt"


@pytest.fixture
def config_file_path() -> str:
    return "tests/AIWolfNLAgentPython/res/config.ini"


# @pytest.fixture
# def agent() -> Agent:
#     config_path = "tests/AIWolfNLAgentPython/res/config.ini"
#     inifile = util.read_config_file(config_file_path=config_path)

#     return Agent(inifile=inifile, name="test")


# @pytest.fixture
# def agent_villager(villager_text: str) -> Villager:
#     config_path = "tests/AIWolfNLAgentPython/res/config.ini"
#     inifile = util.read_config_file(config_file_path=config_path)

#     agent = Villager(inifile=inifile, name="test_villager")

#     agent.parse_info(receive=villager_text)
#     agent.get_info()
#     agent.action()

#     return agent


# @pytest.fixture
# def agent_seer(seer_text: str) -> Seer:
#     config_path = "tests/AIWolfNLAgentPython/res/config.ini"
#     inifile = util.read_config_file(config_file_path=config_path)

#     agent = Seer(inifile=inifile, name="test_seer")

#     agent.parse_info(receive=seer_text)
#     agent.get_info()
#     agent.action()

#     return agent


# @pytest.fixture
# def agent_werewolf(werewolf_text: str) -> Werewolf:
#     config_path = "tests/AIWolfNLAgentPython/res/config.ini"
#     inifile = util.read_config_file(config_file_path=config_path)

#     agent = Werewolf(inifile=inifile, name="test_werewolf")

#     agent.parse_info(receive=werewolf_text)
#     agent.get_info()
#     agent.action()

#     return agent


# @pytest.fixture
# def agent_possessed(possessed_text: str) -> Possessed:
#     config_path = "tests/AIWolfNLAgentPython/res/config.ini"
#     inifile = util.read_config_file(config_file_path=config_path)

#     agent = Possessed(inifile=inifile, name="test_possessed")

#     agent.parse_info(receive=possessed_text)
#     agent.get_info()
#     agent.action()

#     return agent



import json
import re

import pytest


def json_fix(json_str: str) -> str:
    json_str = json_str.replace("'", '"')
    json_str = re.sub('[a-zA-Z]"[a-zA-Z]', "'", json_str)
    json_str = json_str.replace("False", '"False"')
    json_str = json_str.replace("True", '"True"')
    json_str = json_str.replace("None", '"None"')

    return json_str


@pytest.fixture
def status_map_json() -> None:
    status_map_json = """{"request":"ATTACK","gameInfo":{"day":2,"agent":"Agent[04]","executedAgent":"Agent[01]","latestExecutedAgent":"Agent[03]","attackedAgent":"Agent[02]","voteList":[],"latestVoteList":[],"attackVoteList":[{"day":1,"agent":"Agent[04]","target":"Agent[02]"}],"latestAttackVoteList":[],"talkList":[{"idx":17,"day":2,"turn":5,"agent":"Agent[04]","text":"Over","skip":false,"over":true}],"whisperList":[],"statusMap":{"Agent[01]":"DEAD","Agent[02]":"DEAD","Agent[03]":"DEAD","Agent[04]":"ALIVE","Agent[05]":"ALIVE"},"roleMap":{"Agent[04]":"WEREWOLF"},"remainTalkMap":{"Agent[03]":0,"Agent[04]":0,"Agent[05]":5},"remainWhisperMap":{"Agent[04]":5},"existingRoleList":["POSSESSED","SEER","VILLAGER","WEREWOLF"],"lastDeadAgentList":["Agent[02]"]}}"""
    return json.loads(status_map_json)



from aiwolf_nlp_common.action import Action


def check_is_action(
    target_json: dict,
    is_name: bool = False,
    is_initialize: bool = False,
    is_daily_initialize: bool = False,
    is_talk: bool = False,
    is_daily_finish: bool = False,
    is_divine: bool = False,
    is_vote: bool = False,
    is_attack: bool = False,
    is_finish: bool = False,
) -> None:
    request = target_json["request"]

    if is_name:
        assert Action.is_name(request=request)
    else:
        assert not Action.is_name(request=request)

    if is_initialize:
        assert Action.is_initialize(request=request)
    else:
        assert not Action.is_initialize(request=request)

    if is_daily_initialize:
        assert Action.is_daily_initialize(request=request)
    else:
        assert not Action.is_daily_initialize(request=request)

    if is_talk:
        assert Action.is_talk(request=request)
    else:
        assert not Action.is_talk(request=request)

    if is_daily_finish:
        assert Action.is_daily_finish(request=request)
    else:
        assert not Action.is_daily_finish(request=request)

    if is_divine:
        assert Action.is_divine(request=request)
    else:
        assert not Action.is_divine(request=request)

    if is_vote:
        assert Action.is_vote(request=request)
    else:
        assert not Action.is_vote(request=request)

    if is_attack:
        assert Action.is_attack(request=request)
    else:
        assert not Action.is_attack(request=request)

    if is_finish:
        assert Action.is_finish(request=request)
    else:
        assert not Action.is_finish(request=request)


def test_name(name_json) -> None:
    check_is_action(target_json=name_json, is_name=True)


def test_initialize(initialize_json) -> None:
    check_is_action(target_json=initialize_json, is_initialize=True)


def test_daily_initialize(daily_initialize_json) -> None:
    check_is_action(target_json=daily_initialize_json, is_daily_initialize=True)


def test_talk(talk_json) -> None:
    check_is_action(target_json=talk_json, is_talk=True)


def test_daily_finish(daily_finish_json) -> None:
    check_is_action(target_json=daily_finish_json, is_daily_finish=True)


def test_divine(divine_json) -> None:
    check_is_action(target_json=divine_json, is_divine=True)


def test_vote(vote_json) -> None:
    check_is_action(target_json=vote_json, is_vote=True)


def test_attack(attack_json) -> None:
    check_is_action(target_json=attack_json, is_attack=True)


def test_finish(finish_json) -> None:
    check_is_action(target_json=finish_json, is_finish=True)



import socket

from aiwolf_nlp_common.client.client import Client


class TCPClient(Client):
    def __init__(
        self,
        host_ip: str,
        host_port: int,
    ) -> None:
        super().__init__()
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.host_ip = host_ip
        self.host_port = host_port

    def connect(self) -> None:
        self.socket.connect((self.host_ip, self.host_port))

    def receive(self) -> str:
        resp = b""
        while not self.is_valid(resp=resp):
            raw = self.socket.recv(2048)
            if raw == b"":
                err_message = "socket connection broken"
                raise RuntimeError(err_message)
            resp += raw
        return resp.decode(self.encode)

    def send(self, req: str) -> None:
        if not req.endswith("\n"):
            req += "\n"
        self.socket.send(req.encode(self.encode))

    def close(self) -> None:
        self.socket.close()



from __future__ import annotations

import enum


class Status(enum.Enum):
    ALIVE = "ALIVE"
    DEAD = "DEAD"

    @classmethod
    def is_alive(cls, status: str | Status) -> bool:
        return status in {cls.ALIVE, cls.ALIVE.value}

    @classmethod
    def is_dead(cls, status: str | Status) -> bool:
        return status in {cls.DEAD, cls.DEAD.value}



from aiwolf_nlp_common.protocol.gameInfo.list.last_dead_agent_list import LastDeadAgentList


def test_set_received_info(initialize_json: dict, last_dead_agent_list_json: dict) -> None:
    test_list = LastDeadAgentList.initialize_from_json(
        set_list=initialize_json["gameInfo"]["lastDeadAgentList"]
    )

    assert test_list == []

    test_list = LastDeadAgentList.initialize_from_json(
        set_list=last_dead_agent_list_json["gameInfo"]["lastDeadAgentList"]
    )
    result: list = ["Agent[02]"]

    assert len(test_list) == len(result)
    assert all(x == y and type(x) == type(y) for x, y in zip(test_list, result))



from aiwolf_nlp_common.protocol.info import Info
from aiwolf_nlp_common.role import RoleInfo


def test_info(initialize_json):
    info_key = "info"
    info = Info.initialize_from_json(value=initialize_json[info_key])

    assert initialize_json[info_key]["day"] == info.day
    assert initialize_json[info_key]["agent"] == info.agent
    assert info.medium_result.is_empty()
    assert not info.has_executed_agent()
    assert not info.has_attacked_agent()
    assert info.vote_list.is_empty()
    assert info.attack_vote_list.is_empty()
    assert info.status_map.get_alive_agent_list() == [
        "Agent[01]",
        "Agent[02]",
        "Agent[03]",
        "Agent[04]",
        "Agent[05]",
    ]
    assert info.role_map.get_role(agent="Agent[01]") == RoleInfo.VILLAGER.value



from aiwolf_nlp_common.protocol.setting import Setting
from aiwolf_nlp_common.role import RoleInfo


def test_info(initialize_json):
    setting_key = "setting"
    setting = Setting.initialize_from_json(value=initialize_json[setting_key])

    assert setting.role_num_map.get_role_num(role=RoleInfo.get_villager_ja()) == 2
    assert (
        setting.role_num_map.get_role_num(role=RoleInfo.VILLAGER)
        == initialize_json[setting_key]["roleNumMap"]["VILLAGER"]
    )
    assert (
        setting.role_num_map.get_role_num(role=RoleInfo.VILLAGER.value)
        == initialize_json[setting_key]["roleNumMap"]["VILLAGER"]
    )
    assert setting.max_talk == initialize_json[setting_key]["maxTalk"]
    assert setting.max_talk_turn == initialize_json[setting_key]["maxTalkTurn"]
    assert setting.max_whisper == initialize_json[setting_key]["maxWhisper"]
    assert setting.max_whisper_turn == initialize_json[setting_key]["maxWhisperTurn"]
    assert setting.max_skip == initialize_json[setting_key]["maxSkip"]
    assert setting.is_enable_no_attack == bool(initialize_json[setting_key]["isEnableNoAttack"])
    assert setting.is_vote_visible == bool(initialize_json[setting_key]["isVoteVisible"])
    assert setting.is_talk_on_first_day == bool(initialize_json[setting_key]["isTalkOnFirstDay"])
    assert setting.response_timeout == Setting.convert_ms_to_seconds(
        time=initialize_json[setting_key]["responseTimeout"]
    )
    assert setting.action_timeout == Setting.convert_ms_to_seconds(
        time=initialize_json[setting_key]["actionTimeout"]
    )
    assert setting.max_revote == initialize_json[setting_key]["maxRevote"]
    assert setting.max_attack_revote == initialize_json[setting_key]["maxAttackRevote"]



from __future__ import annotations

from typing import TYPE_CHECKING

import paramiko

from aiwolf_nlp_common.client import Client

if TYPE_CHECKING:
    from pathlib import Path


class SSHClient(Client):
    def __init__(  # noqa: PLR0913
        self,
        ssh_config_path: Path,
        ssh_host_name: str,
        ssh_username: str,
        ssh_key_filename: str,
        use_ssh_agent: bool,  # noqa: FBT001
        timeout: int,
        remote_forward_port: int,
    ) -> None:
        super().__init__()
        self.ssh_config_path = ssh_config_path
        self.ssh_host_name = ssh_host_name
        self.ssh_username = ssh_username
        self.ssh_key_filename = ssh_key_filename
        self.use_ssh_agent = use_ssh_agent
        self.timeout = timeout
        self.remote_forward_port = remote_forward_port

    def read_ssh_config(self) -> paramiko.SSHConfigDict:
        config_file = self.ssh_config_path.expanduser()
        ssh_config = paramiko.SSHConfig()
        with config_file.open("r", encoding=self.encode) as file:
            ssh_config.parse(file)
        return ssh_config.lookup(self.ssh_host_name)

    def set_ssh_toolkit(self) -> None:
        self.ssh_client = paramiko.SSHClient()
        self.ssh_agent = paramiko.Agent()
        self.ssh_agent_keys = self.ssh_agent.get_keys()
        self.ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

    def set_ssh_config(self) -> None | ValueError:
        self.config = self.read_ssh_config()
        self.ssh_pkey = None
        self.key_filename = None
        if self.use_ssh_agent and len(self.ssh_agent_keys) > 0:
            self.ssh_pkey = self.ssh_agent_keys[0]
        elif self.use_ssh_agent and len(self.ssh_agent_keys) == 0:
            err_message = (
                "ssh-agent is not available or no keys are available in ssh-agent"
            )
            raise ValueError(err_message)

    def connect(self) -> None:
        self.set_ssh_toolkit()
        self.set_ssh_config()
        self.ssh_client.connect(
            hostname=self.ssh_host_name,
            username=self.ssh_username,
            pkey=self.ssh_pkey,
            key_filename=self.ssh_key_filename,
            timeout=self.timeout,
        )
        self.transport = self.ssh_client.get_transport()
        self.ssh_remote_forward()

    def ssh_remote_forward(self) -> None:
        if self.transport is None:
            err_message = "transport is not connected"
            raise RuntimeError(err_message)
        self.transport.request_port_forward(address="", port=self.remote_forward_port)
        self.channel = self.transport.accept()

    def receive(self) -> str:
        if self.channel is None:
            err_message = "channel is not connected"
            raise RuntimeError(err_message)
        resp = b""
        while not self.is_valid(resp=resp):
            raw = self.channel.recv(2048)
            if raw == b"":
                err_message = "socket connection broken"
                raise RuntimeError(err_message)
            resp += raw
        return resp.decode(self.encode)

    def send(self, req: str) -> None:
        if not req.endswith("\n"):
            req += "\n"
        if self.channel is not None:
            self.channel.send(bytes(req, self.encode))

    def close(self) -> None:
        self.ssh_client.close()
        if self.channel is not None:
            self.channel.close()



from __future__ import annotations

from aiwolf_nlp_common.status import Status


class AgentStatus:
    def __init__(self, agent: str, status: str) -> None:
        self.__agent = agent
        self.__status = Status.ALIVE if Status.is_alive(status=status) else Status.DEAD

    def __hash__(self) -> int:
        return hash((self.agent, self.status.value))

    def __eq__(self, value: object) -> bool:
        if value is None or not isinstance(value, self.__class__):
            return NotImplemented
        return self.agent == value.agent and self.status.value == value.status.value

    def __lt__(self, value: object) -> bool:
        if not isinstance(value, self.__class__):
            return NotImplemented
        return self.__agent < value.__agent

    def __str__(self) -> str:
        return f"{self.__agent} is {self.status.value}"

    @property
    def agent(self) -> str:
        return self.__agent

    @property
    def status(self) -> Status:
        return self.__status


class StatusMap(set):
    def __str__(self) -> str:
        output: str = f"[{self.__class__.__name__}]"
        if self.is_empty():
            return output + "\nNo Result Available"
        return output + "\n".join(str(elem) for elem in sorted(self))

    @classmethod
    def initialize_from_json(cls, value: dict | None = None) -> StatusMap:
        instance = cls()
        if value is None:
            return instance
        for agent, status in value.items():
            instance.add(AgentStatus(agent=agent, status=status))
        return instance

    def __init__(self, value: dict | None = None) -> None:
        self.initialize_from_json(value)

    def is_empty(self) -> bool:
        return len(self) == 0

    def reverse_status(self, agent: str) -> None:
        prev_alive = AgentStatus(agent=agent, status=Status.ALIVE.value)
        prev_dead = AgentStatus(agent=agent, status=Status.DEAD.value)
        if prev_alive not in self and prev_dead not in self:
            raise ValueError(agent, "is a name that does not exist.")
        if prev_alive in self:
            self.remove(prev_alive)
            self.add(prev_dead)
        else:
            self.remove(prev_dead)
            self.add(prev_alive)

    def set_alive(self, agent: str) -> None:
        self.add(AgentStatus(agent=agent, status=Status.ALIVE.value))

    def set_dead(self, agent: str) -> None:
        self.add(AgentStatus(agent=agent, status=Status.DEAD.value))

    def get_alive_agent_list(self) -> list:
        return sorted(
            [status.agent for status in self if Status.is_alive(status=status.status)],
        )

    def get_dead_agent_list(self) -> list:
        return sorted(
            [status.agent for status in self if Status.is_dead(status=status.status)],
        )



import pytest

from aiwolf_nlp_common.protocol.gameInfo.map.role_map import AgentRole, RoleMap
from aiwolf_nlp_common.protocol.gameSetting.map.role_num_map import RoleNumMap
from aiwolf_nlp_common.role.role import RoleInfo


def test_get_agent_list(initialize_json: dict) -> None:
    test_map = RoleNumMap(value=initialize_json["gameSetting"]["roleNumMap"])

    assert test_map.get_role_num(role=RoleInfo.VILLAGER.value) == 2
    assert test_map.get_role_num(role="SEER") == 1
    assert test_map.get_role_num(role=RoleInfo.WEREWOLF) == 1

    with pytest.raises(ValueError) as e:
        test_map.get_role_num(role="Seer")



from __future__ import annotations

from dataclasses import dataclass


@dataclass(frozen=True)
class VoteInfo:
    agent: int
    day: int
    target: int

    def __str__(self) -> str:
        return f"Day {self.day}: {self.agent} -> {self.target}"


class VoteList(list[VoteInfo]):
    def __str__(self) -> str:
        output: str = f"[{self.__class__.__name__}]"
        return (
            f"{output}\nNo Result Available"
            if self.is_empty()
            else f"{output}\n" + "\n".join(map(str, self))
        )

    @classmethod
    def initialize_from_json(cls, value: list[dict] | None = None) -> VoteList:
        instance = cls()
        if value is None:
            return instance
        instance.extend(
            VoteInfo(
                agent=vote_info["agent"],
                day=vote_info["day"],
                target=vote_info["target"],
            )
            for vote_info in value
        )
        return instance

    def __init__(self, value: list[dict] | None = None) -> None:
        self.initialize_from_json(value)

    def is_empty(self) -> bool:
        return not bool(self)



from __future__ import annotations

from dataclasses import dataclass
from typing import ClassVar


@dataclass
class JudgementResult:
    DEFAULT_DAY: ClassVar[int] = -1
    DEFAULT_AGENT: ClassVar[str] = "Agent[00]"
    DEFAULT_TARGET: ClassVar[str] = "Agent[00]"
    DEFAULT_RESULT: ClassVar[str] = "unknown"

    day: int = DEFAULT_DAY
    agent: str = DEFAULT_AGENT
    target: str = DEFAULT_TARGET
    result: str = DEFAULT_RESULT

    def __str__(self) -> str:
        if self.is_empty():
            return f"[{self.__class__.__name__}]\nNo Result Available"
        return f"[{self.__class__.__name__}]\nDay {self.day} : {self.target} is {self.result}"

    @classmethod
    def initialize_from_json(cls, value: dict | None) -> JudgementResult:
        if value is None:
            return cls()
        return cls(**value)

    def __init__(self, value: dict | None = None) -> None:
        self.initialize_from_json(value)

    def reset(self) -> None:
        self.day = self.DEFAULT_DAY
        self.agent = self.DEFAULT_AGENT
        self.target = self.DEFAULT_TARGET
        self.result = self.DEFAULT_RESULT

    def is_empty(self) -> bool:
        return (
            self.day == self.DEFAULT_DAY
            and self.agent == self.DEFAULT_AGENT
            and self.target == self.DEFAULT_TARGET
            and self.result == self.DEFAULT_RESULT
        )



import json
import re

import pytest


def json_fix(json_str: str) -> str:
    json_str = json_str.replace("'", '"')
    json_str = re.sub('[a-zA-Z]"[a-zA-Z]', "'", json_str)
    json_str = json_str.replace("False", '"False"')
    json_str = json_str.replace("True", '"True"')
    json_str = json_str.replace("None", '"None"')

    return json_str


@pytest.fixture
def vote_list_json() -> dict:
    vote_list_json = """{'gameInfo': {'agent': 4, 'attackVoteList': [], 'attackedAgent': -1, 'cursedFox': -1, 'day': 3, 'divineResult': None, 'englishTalkList': [], 'executedAgent': 1, 'existingRoleList': ['POSSESSED', 'SEER', 'VILLAGER', 'WEREWOLF'], 'guardedAgent': -1, 'lastDeadAgentList': [], 'latestAttackVoteList': [], 'latestExecutedAgent': -1, 'latestVoteList': [], 'mediumResult': None, 'remainTalkMap': {'2': 10, '5': 10}, 'remainWhisperMap': {}, 'roleMap': {'1': 'WEREWOLF', '2': 'SEER', '3': 'POSSESSED', '4': 'VILLAGER', '5': 'VILLAGER'}, 'statusMap': {'1': 'DEAD', '2': 'ALIVE', '3': 'DEAD', '4': 'DEAD', '5': 'ALIVE'}, 'talkList': [], 'voteList': [{'agent': 1, 'day': 2, 'target': 2}, {'agent': 2, 'day': 2, 'target': 1}, {'agent': 5, 'day': 2, 'target': 1}], 'whisperList': []}, 'gameSetting': None, 'request': 'FINISH', 'talkHistory': None, 'whisperHistory': None}\n"""
    return json.loads(json_fix(json_str=vote_list_json))


@pytest.fixture
def talk_list_json() -> dict:
    talk_list_json = """{"request":"ATTACK","gameInfo":{"day":2,"agent":"Agent[04]","executedAgent":"Agent[01]","latestExecutedAgent":"Agent[03]","attackedAgent":"Agent[02]","voteList":[],"latestVoteList":[],"attackVoteList":[{"day":1,"agent":"Agent[04]","target":"Agent[02]"}],"latestAttackVoteList":[],"talkList":[{"idx":17,"day":2,"turn":5,"agent":"Agent[04]","text":"Over","skip":false,"over":true},{"idx":18,"day":2,"turn":5,"agent":"Agent[04]","text":"Skip","skip":true,"over":false},{"idx":19,"day":2,"turn":5,"agent":"Agent[04]","text":"Test","skip":false,"over":false}],"whisperList":[],"statusMap":{"Agent[01]":"DEAD","Agent[02]":"DEAD","Agent[03]":"DEAD","Agent[04]":"ALIVE","Agent[05]":"ALIVE"},"roleMap":{"Agent[04]":"WEREWOLF"},"remainTalkMap":{"Agent[03]":0,"Agent[04]":0,"Agent[05]":5},"remainWhisperMap":{"Agent[04]":5},"existingRoleList":["POSSESSED","SEER","VILLAGER","WEREWOLF"],"lastDeadAgentList":["Agent[02]"]}}"""
    return json.loads(talk_list_json)


@pytest.fixture
def existing_role_list_json() -> dict:
    existing_role_list_json = """{'gameInfo': {'agent': 4, 'attackVoteList': [], 'attackedAgent': -1, 'cursedFox': -1, 'day': 2, 'divineResult': None, 'englishTalkList': [], 'executedAgent': 3, 'existingRoleList': ['POSSESSED', 'SEER', 'VILLAGER', 'WEREWOLF'], 'guardedAgent': -1, 'lastDeadAgentList': [4], 'latestAttackVoteList': [], 'latestExecutedAgent': -1, 'latestVoteList': [], 'mediumResult': None, 'remainTalkMap': {'1': 10, '2': 10, '5': 10}, 'remainWhisperMap': {}, 'roleMap': {'4': 'VILLAGER'}, 'statusMap': {'1': 'ALIVE', '2': 'ALIVE', '3': 'DEAD', '4': 'DEAD', '5': 'ALIVE'}, 'talkList': [], 'voteList': [{'agent': 1, 'day': 1, 'target': 3}, {'agent': 2, 'day': 1, 'target': 4}, {'agent': 3, 'day': 1, 'target': 2}, {'agent': 4, 'day': 1, 'target': 3}, {'agent': 5, 'day': 1, 'target': 4}], 'whisperList': []}, 'gameSetting': {'enableNoAttack': False, 'enableNoExecution': False, 'enableRoleRequest': False, 'maxAttackRevote': 1, 'maxRevote': 1, 'maxSkip': 0, 'maxTalk': 10, 'maxTalkTurn': 20, 'maxWhisper': 10, 'maxWhisperTurn': 20, 'playerNum': 5, 'randomSeed': 1710509577819, 'roleNumMap': {'MEDIUM': 0, 'WEREWOLF': 1, 'VILLAGER': 2, 'FOX': 0, 'SEER': 1, 'FREEMASON': 0, 'POSSESSED': 1, 'BODYGUARD': 0}, 'talkOnFirstDay': True, 'timeLimit': 6000000, 'validateUtterance': False, 'votableInFirstDay': False, 'voteVisible': True, 'whisperBeforeRevote': False}, 'request': 'DAILY_INITIALIZE', 'talkHistory': None, 'whisperHistory': None}\n"""
    return json.loads(existing_role_list_json)


@pytest.fixture
def last_dead_agent_list_json() -> dict:
    last_dead_agent_list_json = """{"request":"ATTACK","gameInfo":{"day":2,"agent":"Agent[04]","executedAgent":"Agent[01]","latestExecutedAgent":"Agent[03]","attackedAgent":"Agent[02]","voteList":[],"latestVoteList":[],"attackVoteList":[{"day":1,"agent":"Agent[04]","target":"Agent[02]"}],"latestAttackVoteList":[],"talkList":[{"idx":17,"day":2,"turn":5,"agent":"Agent[04]","text":"Over","skip":false,"over":true}],"whisperList":[],"statusMap":{"Agent[01]":"DEAD","Agent[02]":"DEAD","Agent[03]":"DEAD","Agent[04]":"ALIVE","Agent[05]":"ALIVE"},"roleMap":{"Agent[04]":"WEREWOLF"},"remainTalkMap":{"Agent[03]":0,"Agent[04]":0,"Agent[05]":5},"remainWhisperMap":{"Agent[04]":5},"existingRoleList":["POSSESSED","SEER","VILLAGER","WEREWOLF"],"lastDeadAgentList":["Agent[02]"]}}"""

    return json.loads(last_dead_agent_list_json)



from aiwolf_nlp_common.protocol.gameInfo.map.status_map import AgentStatus, Status, StatusMap


def test_set_received_info(initialize_json: dict, status_map_json: dict) -> None:
    test_map = StatusMap.initialize_from_json(set_map=initialize_json["gameInfo"]["statusMap"])

    check_set: set[AgentStatus] = set()
    for i in range(5):
        name = "Agent[0" + str(i + 1) + "]"
        add_elem = AgentStatus(agent=name, status="ALIVE")
        check_set.add(add_elem)

    assert test_map == check_set

    test_map = StatusMap.initialize_from_json(set_map=status_map_json["gameInfo"]["statusMap"])
    check_set.clear()

    for i in range(5):
        name = "Agent[0" + str(i + 1) + "]"

        if i + 1 != 1 and i + 1 != 2 and i + 1 != 3:
            check_set.add(AgentStatus(agent=name, status="ALIVE"))
        else:
            check_set.add(AgentStatus(agent=name, status="DEAD"))

    assert test_map == check_set


def test_set_alive() -> None:
    test_map = StatusMap()

    check_set: set[AgentStatus] = set()
    for i in range(5):
        name = "Agent[0" + str(i + 1) + "]"

        test_map.set_alive(agent=name)
        add_elem = AgentStatus(agent=name, status="ALIVE")
        check_set.add(add_elem)

    assert test_map == check_set


def test_set_dead() -> None:
    test_map = StatusMap()

    check_set: set[AgentStatus] = set()

    for i in range(5):
        name = "Agent[0" + str(i + 1) + "]"

        if i + 1 != 1 and i + 1 != 2 and i + 1 != 3:
            check_set.add(AgentStatus(agent=name, status="ALIVE"))
            test_map.set_alive(agent=name)
        else:
            check_set.add(AgentStatus(agent=name, status="DEAD"))
            test_map.set_dead(agent=name)

    assert test_map == check_set


def test_set_reverse_status() -> None:
    test_map = StatusMap()

    check_set: set[AgentStatus] = set()

    for i in range(5):
        name = "Agent[0" + str(i + 1) + "]"

        if i + 1 != 1 and i + 1 != 2 and i + 1 != 3:
            check_set.add(AgentStatus(agent=name, status="ALIVE"))
            test_map.set_dead(agent=name)
        else:
            check_set.add(AgentStatus(agent=name, status="DEAD"))
            test_map.set_alive(agent=name)

        test_map.reverse_status(agent=name)

    assert test_map == check_set


def test_get_agent_list(status_map_json: dict) -> None:
    test_map = StatusMap.initialize_from_json(set_map=status_map_json["gameInfo"]["statusMap"])

    alive_list = ["Agent[04]", "Agent[05]"]
    dead_list = ["Agent[01]", "Agent[02]", "Agent[03]"]

    assert test_map.get_alive_agent_list() == alive_list
    assert test_map.get_dead_agent_list() == dead_list



from __future__ import annotations

from aiwolf_nlp_common.role import Role, RoleInfo


class RoleNumInfo:
    __role: Role
    __allocated_count: int

    def __init__(self, role: str, allocated_count: int) -> None:
        self.__role = RoleInfo.get_role_info(role=role)
        self.__allocated_count = allocated_count

    def __hash__(self) -> int:
        return hash((self.__role.en, self.__allocated_count))

    def __eq__(self, value: object) -> bool:
        if value is None or not isinstance(value, RoleNumInfo):
            return False
        return (
            self.__role == value.role
            and self.__allocated_count == value.allocated_count
        )

    def __lt__(self, value: object) -> bool:
        if value is None or not isinstance(value, self.__class__):
            return NotImplemented
        return self.__role.en < value.__role.en

    def __str__(self) -> str:
        return f"{self.__role.en} : {self.__allocated_count}"

    @property
    def role(self) -> Role:
        return self.__role

    @property
    def allocated_count(self) -> int:
        return self.__allocated_count


class RoleNumMap(set):
    def __str__(self) -> str:
        output: str = f"[{self.__class__.__name__}]"
        if self.is_empty():
            return output + "\nNo Result Available"
        output_list = list(self)
        elem: RoleNumInfo
        for elem in sorted(output_list):
            output += "\n" + elem.__str__()
        return output

    @classmethod
    def initialize_from_json(cls, value: dict | None = None) -> RoleNumMap:
        instance = cls()
        if value is None:
            return instance
        for role, count in value.items():
            instance.add(RoleNumInfo(role=role, allocated_count=count))
        return instance

    def __init__(self, value: dict | None = None) -> None:
        self.initialize_from_json(value)

    def get_role_num(self, role: str | Role) -> int:
        if not RoleInfo.is_exist_role(role=role):
            raise ValueError(role, "is a name that does not exist.")
        if not isinstance(role, Role):
            role = RoleInfo.get_role_info(role=role)
        for role_num_info in self:
            if role_num_info.role == role:
                return role_num_info.allocated_count
        raise ValueError(role, "is a name that does not exist.")

    def is_empty(self) -> bool:
        return len(self) == 0



from aiwolf_nlp_common.protocol.gameInfo.list.vote_list import VoteInfo, VoteList


def test_set_received_info(initialize_json: dict, vote_list_json: dict) -> None:
    vote_list = VoteList.initialize_from_json(set_list=initialize_json["gameInfo"]["voteList"])

    assert vote_list == []

    vote_list = VoteList.initialize_from_json(set_list=vote_list_json["gameInfo"]["voteList"])
    result: list[VoteInfo] = []

    for temp in vote_list_json["gameInfo"]["voteList"]:
        add_elem = VoteInfo(agent=temp["agent"], day=temp["day"], target=temp["target"])
        result.append(add_elem)

    assert len(vote_list) == len(result)
    assert all(x == y and type(x) == type(y) for x, y in zip(vote_list, result))



from aiwolf_nlp_common.protocol.talk_list import TalkInfo, TalkList


def test_set_received_info(initialize_json: dict, talk_list_json: dict) -> None:
    talk_list = TalkList.initialize_from_json(set_list=initialize_json["gameInfo"]["talkList"])

    assert talk_list == []

    talk_list = TalkList.initialize_from_json(set_list=talk_list_json["gameInfo"]["talkList"])
    result: list[TalkInfo] = []

    for temp in talk_list_json["gameInfo"]["talkList"]:
        add_elem = TalkInfo(
            agent=temp["agent"],
            day=temp["day"],
            idx=temp["idx"],
            text=temp["text"],
            turn=temp["turn"],
            skip=temp["skip"],
            over=temp["over"],
        )
        result.append(add_elem)

    assert len(talk_list) == len(result)
    assert all(x == y and type(x) == type(y) for x, y in zip(talk_list, result))

    talk: TalkInfo
    for talk in talk_list:
        if talk.text == "Skip":
            assert talk.is_skip() and not talk.is_over()
        elif talk.text == "Over":
            assert not talk.is_skip() and talk.is_over()
        else:
            assert not talk.is_skip() and not talk.is_over()



import enum


class Judgement(enum.Enum):
    HUMAN = "HUMAN"
    WEREWOLF = "WEREWOLF"

    @classmethod
    def is_human(cls, result: str) -> bool:
        return result in {cls.HUMAN, cls.HUMAN.value}

    @classmethod
    def is_werewolf(cls, result: str) -> bool:
        return result in {cls.WEREWOLF, cls.WEREWOLF.value}



from __future__ import annotations

from aiwolf_nlp_common.protocol.info.result import DivineResult, MediumResult

from .list import AttackVoteList, VoteList
from .map.role_map import RoleMap
from .map.status_map import StatusMap


class Info:
    day: int
    agent: str
    medium_result: MediumResult
    divine_result: DivineResult
    executed_agent: str | None
    attacked_agent: str | None
    vote_list: VoteList
    attack_vote_list: AttackVoteList
    status_map: StatusMap
    role_map: RoleMap

    def __init__(  # noqa: PLR0913
        self,
        day: int,
        agent: str,
        medium_result: MediumResult,
        divine_result: DivineResult,
        executed_agent: str | None,
        attacked_agent: str | None,
        vote_list: VoteList,
        attack_vote_list: AttackVoteList,
        status_map: StatusMap,
        role_map: RoleMap,
    ) -> None:
        self.day = day
        self.agent = agent
        self.medium_result = medium_result
        self.divine_result = divine_result
        self.executed_agent = executed_agent
        self.attacked_agent = attacked_agent
        self.vote_list = vote_list
        self.attack_vote_list = attack_vote_list
        self.status_map = status_map
        self.role_map = role_map

    def __str__(self) -> str:
        executed_text = (
            self.executed_agent if self.has_executed_agent() else "No Result Available"
        )
        attacked_text = (
            self.attacked_agent if self.has_attacked_agent() else "No Result Available"
        )

        return (
            f"Day: {self.day}\n"
            f"Agent: {self.agent}\n\n"
            f"{self.medium_result}\n\n"
            f"{self.divine_result}\n\n"
            f"Executed Agent: {executed_text}\n"
            f"Attacked Agent: {attacked_text}\n\n"
            f"{self.status_map}\n\n"
            f"{self.role_map}\n\n"
        )

    @classmethod
    def initialize_from_json(cls, value: dict) -> Info:
        return cls(
            day=value["day"],
            agent=value["agent"],
            medium_result=MediumResult(value.get("mediumResult")),
            divine_result=DivineResult(value.get("divineResult")),
            executed_agent=value.get("executedAgent"),
            attacked_agent=value.get("attackedAgent"),
            vote_list=VoteList(value.get("voteList")),
            attack_vote_list=AttackVoteList(value.get("attackVoteList")),
            status_map=StatusMap(value["statusMap"]),
            role_map=RoleMap(value["roleMap"]),
        )

    def update_from_json(self, value: dict | None) -> None:
        if value is None:
            return
        self.day = value["day"]
        self.agent = value["agent"]
        self.medium_result = MediumResult(value.get("mediumResult"))
        self.divine_result = DivineResult(value.get("divineResult"))
        self.executed_agent = value.get("executedAgent")
        self.attacked_agent = value.get("attackedAgent")
        self.vote_list = VoteList(value.get("voteList"))
        self.attack_vote_list = AttackVoteList(value.get("attackVoteList"))
        self.status_map = StatusMap(value["statusMap"])
        self.role_map = RoleMap(value["roleMap"])

    def has_executed_agent(self) -> bool:
        return self.executed_agent is not None

    def has_attacked_agent(self) -> bool:
        return self.attacked_agent is not None



from __future__ import annotations

from aiwolf_nlp_common.role import Role, RoleInfo


class AgentRole:
    __agent: str
    __role: Role

    def __init__(self, agent: str, role: str) -> None:
        self.__agent = agent
        self.__role = RoleInfo.get_role_info(role=role)

    def __hash__(self) -> int:
        return hash((self.agent, self.role.en))

    def __eq__(self, value: object) -> bool:
        if value is None or not isinstance(value, AgentRole):
            return False
        return self.agent == value.agent and self.role == value.role

    def __lt__(self, value: object) -> bool:
        if value is None or not isinstance(value, self.__class__):
            return NotImplemented
        return self.__agent < value.__agent

    def __str__(self) -> str:
        return f"{self.__agent} is {self.__role.en}"

    @property
    def agent(self) -> str:
        return self.__agent

    @property
    def role(self) -> Role:
        return self.__role


class RoleMap(set):
    def __str__(self) -> str:
        output: str = f"[{self.__class__.__name__}]"
        if self.is_empty():
            return output + "\nNo Result Available"
        return output + "\n" + "\n".join(str(elem) for elem in sorted(self))

    @classmethod
    def initialize_from_json(cls, value: dict | None = None) -> RoleMap:
        instance = cls()
        if value is None:
            return instance
        for agent, role in value.items():
            instance.add(AgentRole(agent=agent, role=role))
        return instance

    def __init__(self, value: dict | None = None) -> None:
        self.initialize_from_json(value)

    def get_role(self, agent: str) -> Role:
        for agent_role in self:
            if agent_role.agent == agent:
                return agent_role.role
        raise ValueError(agent, "is a name that does not exist.")

    def is_empty(self) -> bool:
        return len(self) == 0



import pytest

from aiwolf_nlp_common.protocol.gameInfo.map.role_map import AgentRole, RoleMap
from aiwolf_nlp_common.role.role import RoleInfo


def test_set_received_info(initialize_json: dict, status_map_json: dict) -> None:
    test_map = RoleMap.initialize_from_json(set_map=initialize_json["gameInfo"]["roleMap"])

    check_set = set()
    check_set.add(AgentRole(agent="Agent[01]", role="VILLAGER"))

    assert test_map == check_set

    test_map = RoleMap.initialize_from_json(set_map=status_map_json["gameInfo"]["roleMap"])
    check_set.clear()
    check_set.add(AgentRole(agent="Agent[04]", role="WEREWOLF"))

    assert test_map == check_set


def test_get_agent_role(initialize_json: dict) -> None:
    test_map = RoleMap.initialize_from_json(set_map=initialize_json["gameInfo"]["roleMap"])

    assert test_map.get_agent_role(agent="Agent[01]") == RoleInfo.VILLAGER.value

    with pytest.raises(ValueError) as e:
        test_map.get_agent_role(agent="Agent[02]")
        test_map.get_agent_role(agent="Agent[03]")
        test_map.get_agent_role(agent="Agent[04]")
        test_map.get_agent_role(agent="Agent[05]")



from __future__ import annotations

import json

from .info.info import Info
from .list import TalkList, WhisperList
from .setting.setting import Setting


class CommunicationProtocol:
    request: str
    info: Info | None
    setting: Setting | None
    talk_history: TalkList | None
    whisper_history: WhisperList | None

    def __init__(
        self,
        request: str,
        info: Info | None,
        setting: Setting | None,
        talk_history: TalkList | None,
        whisper_history: WhisperList | None,
    ) -> None:
        self.request = request
        self.info = info
        self.setting = setting
        self.talk_history = talk_history
        self.whisper_history = whisper_history

    def __str__(self) -> str:
        return (
            f"Request: {self.request}\n\n"
            f"---info---\n"
            f"{self.info}\n"
            f"---setting---\n"
            f"{self.setting}\n"
            f"---talkHistory---\n"
            f"{self.talk_history}\n\n"
            f"---whisperHistory---\n"
            f"{self.whisper_history}\n"
        )

    @classmethod
    def initialize_from_json(cls, value: str) -> CommunicationProtocol:
        received_json: dict = json.loads(value)
        return cls(
            received_json["request"],
            (
                Info.initialize_from_json(value=received_json["info"])
                if received_json.get("info")
                else None
            ),
            (
                Setting.initialize_from_json(value=received_json["setting"])
                if received_json.get("setting")
                else None
            ),
            TalkList(value=received_json.get("talkHistory")),
            WhisperList(value=received_json.get("whisperHistory")),
        )

    def update_from_json(  # noqa: C901
        self,
        value: str,
    ) -> None:
        received_json: dict = json.loads(value)
        self.request = received_json["request"]
        if received_json.get("info") is not None:
            if self.is_info_empty():
                self.info = Info.initialize_from_json(value=received_json["info"])
            elif self.info is not None:
                self.info.update_from_json(value=received_json.get("info"))
        if received_json.get("setting") is not None:
            if self.is_setting_empty():
                self.setting = Setting.initialize_from_json(
                    value=received_json["setting"],
                )
            elif self.setting is not None:
                self.setting.update_from_json(value=received_json.get("setting"))
        if received_json.get("talkHistory") is not None:
            self.talk_history = TalkList(value=received_json.get("talkHistory"))
        elif self.talk_history is not None:
            self.talk_history.clear()
        if received_json.get("whisperHistory") is not None:
            self.whisper_history = WhisperList(
                value=received_json.get("whisperHistory"),
            )
        elif self.whisper_history is not None:
            self.whisper_history.clear()

    def is_info_empty(self) -> bool:
        return self.info is None

    def is_setting_empty(self) -> bool:
        return self.setting is None

    def is_talk_history_empty(self) -> bool:
        return self.talk_history is None

    def is_whisper_history_empty(self) -> bool:
        return self.whisper_history is None



from __future__ import annotations

import websocket

from aiwolf_nlp_common.client import Client


class WebSocketClient(Client):
    def __init__(self, url: str) -> None:
        super().__init__()
        websocket.enableTrace(traceable=False)
        self.socket = websocket.WebSocket()
        self.url = url

    def connect(self) -> None:
        self.socket.connect(self.url)

    def receive(self) -> list[str]:
        resp = self.socket.recv()
        resp_str = ""
        if isinstance(resp, (bytes, bytearray, memoryview)):
            resp_str = bytes(resp).decode(self.encode)
        else:
            resp_str = resp
        return Client.split_receives(receive=resp_str, include_newline=False)

    def send(self, req: str) -> None:
        if not req.endswith("\n"):
            req += "\n"
        self.socket.send(req)

    def close(self) -> None:
        self.socket.close()



from aiwolf_nlp_common.protocol.gameInfo.list.existing_role_list import ExistingRoleList
from aiwolf_nlp_common.role.role import RoleInfo


def test_set_received_info(initialize_json: dict) -> None:
    vote_list = ExistingRoleList.initialize_from_json(
        set_list=initialize_json["gameInfo"]["existingRoleList"]
    )

    check_list = [
        RoleInfo.POSSESSED.value,
        RoleInfo.SEER.value,
        RoleInfo.VILLAGER.value,
        RoleInfo.WEREWOLF.value,
    ]

    assert vote_list == check_list



import enum


class RoleTeam:
    __en: str
    __ja: str

    def __init__(self, en: str, ja: str) -> None:
        self.__en = en
        self.__ja = ja

    def __eq__(self, value: object) -> bool:
        if value is None or not isinstance(value, RoleTeam):
            return False
        return self.en == value.en and self.ja == value.ja

    def __hash__(self) -> int:
        return hash((self.__en, self.__ja))

    @property
    def en(self) -> str:
        return self.__en

    @property
    def ja(self) -> str:
        return self.__ja


class RoleTeamInfo(enum.Enum):
    VILLAGER_TEAM = RoleTeam(en="VILLAGER", ja="村人陣営")
    WEREWOLF_TEAM = RoleTeam(en="WEREWOLF", ja="人狼陣営")
    FOX_TEAM = RoleTeam(en="FOX", ja="妖狐陣営")
    ANY_TEAM = RoleTeam(en="ANY", ja="?陣営")



from __future__ import annotations

from .map.role_num_map import RoleNumMap


class Setting:
    role_num_map: RoleNumMap
    max_talk: int
    max_talk_turn: int
    max_whisper: int
    max_whisper_turn: int
    max_skip: int
    is_enable_no_attack: bool
    is_vote_visible: bool
    is_talk_on_first_day: bool
    response_timeout: int
    action_timeout: int
    max_revote: int
    max_attack_revote: int
    player_num: int

    def __init__(  # noqa: PLR0913
        self,
        role_num_map: RoleNumMap,
        max_talk: int,
        max_talk_turn: int,
        max_whisper: int,
        max_whisper_turn: int,
        max_skip: int,
        is_enable_no_attack: bool,  # noqa: FBT001
        is_vote_visible: bool,  # noqa: FBT001
        is_talk_on_first_day: bool,  # noqa: FBT001
        response_timeout: int,
        action_timeout: int,
        max_revote: int,
        max_attack_revote: int,
        player_num: int,
    ) -> None:
        self.role_num_map = role_num_map
        self.max_talk = max_talk
        self.max_talk_turn = max_talk_turn
        self.max_whisper = max_whisper
        self.max_whisper_turn = max_whisper_turn
        self.max_skip = max_skip
        self.is_enable_no_attack = is_enable_no_attack
        self.is_vote_visible = is_vote_visible
        self.is_talk_on_first_day = is_talk_on_first_day
        self.response_timeout = response_timeout
        self.action_timeout = action_timeout
        self.max_revote = max_revote
        self.max_attack_revote = max_attack_revote
        self.player_num = player_num

    def __str__(self) -> str:
        return (
            f"{self.role_num_map}\n\n"
            f"MaxTalk: {self.max_talk}\n"
            f"MaxTalkTurn: {self.max_talk_turn}\n"
            f"MaxWhisper: {self.max_whisper}\n"
            f"MaxWhisperTurn: {self.max_whisper_turn}\n"
            f"MaxSkip: {self.max_skip}\n"
            f"isEnableNoAttack: {self.is_enable_no_attack}\n"
            f"isVoteVisible: {self.is_vote_visible}\n"
            f"isTalkOnFirstDay: {self.is_talk_on_first_day}\n"
            f"ResponseTimeOut: {self.response_timeout}\n"
            f"ActionTimeOut: {self.action_timeout}\n"
            f"MaxReVote: {self.max_revote}\n"
            f"MaxAttackReVote: {self.max_attack_revote}\n"
            f"PlayerNum: {self.player_num}\n"
        )

    @classmethod
    def initialize_from_json(cls, value: dict) -> Setting:
        return cls(
            role_num_map=RoleNumMap(value=value["roleNumMap"]),
            max_talk=value["maxTalk"],
            max_talk_turn=value["maxTalkTurn"],
            max_whisper=value["maxWhisper"],
            max_whisper_turn=value["maxWhisperTurn"],
            max_skip=value["maxSkip"],
            is_enable_no_attack=value["isEnableNoAttack"],
            is_vote_visible=value["isVoteVisible"],
            is_talk_on_first_day=value["isTalkOnFirstDay"],
            response_timeout=value["responseTimeout"] / 1000,
            action_timeout=value["actionTimeout"] / 1000,
            max_revote=value["maxRevote"],
            max_attack_revote=value["maxAttackRevote"],
            player_num=value["playerNum"],
        )

    def update_from_json(self, value: dict | None) -> None:
        if value is None:
            return
        self.role_num_map = RoleNumMap(value=value["roleNumMap"])
        self.max_talk = value["maxTalk"]
        self.max_talk_turn = value["maxTalkTurn"]
        self.max_whisper = value["maxWhisper"]
        self.max_whisper_turn = value["maxWhisperTurn"]
        self.max_skip = value["maxSkip"]
        self.is_enable_no_attack = value["isEnableNoAttack"]
        self.is_vote_visible = value["isVoteVisible"]
        self.is_talk_on_first_day = value["isTalkOnFirstDay"]
        self.response_timeout = int(value["responseTimeout"]) // 1000
        self.action_timeout = int(value["actionTimeout"]) // 1000
        self.max_revote = value["maxRevote"]
        self.max_attack_revote = value["maxAttackRevote"]
        self.player_num = value["playerNum"]



from __future__ import annotations

from .judgement_result import JudgementResult


class MediumResult(JudgementResult):
    def __init__(self, value: dict | None = None) -> None:
        super().__init__(value)



from __future__ import annotations

import enum

from .role_team import RoleTeam, RoleTeamInfo

LANGUAGE = 2

EN_POS = 1
JA_POS = 2
TEAM_POS = 3


class Role:
    __en: str
    __ja: str
    __team: RoleTeam

    def __init__(self, en: str, ja: str, team: RoleTeam) -> None:
        self.__en = en
        self.__ja = ja
        self.__team = team

    def __eq__(self, value: object) -> bool:
        if value is None or not isinstance(value, Role):
            return False
        return self.en == value.en and self.ja == value.ja and self.team == value.team

    def __hash__(self) -> int:
        return hash((self.__en, self.__ja, self.__team))

    @property
    def en(self) -> str:
        return self.__en

    @property
    def ja(self) -> str:
        return self.__ja

    @property
    def team(self) -> RoleTeam:
        return self.__team

    def get_translations(self) -> set:
        return {self.__en, self.__ja}


class RoleInfo(enum.Enum):
    VILLAGER = Role(en="VILLAGER", ja="村人", team=RoleTeamInfo.VILLAGER_TEAM.value)
    SEER = Role(en="SEER", ja="占い師", team=RoleTeamInfo.VILLAGER_TEAM.value)
    MEDIUM = Role(en="MEDIUM", ja="霊媒師", team=RoleTeamInfo.VILLAGER_TEAM.value)
    BODYGUARD = Role(en="BODYGUARD", ja="騎士", team=RoleTeamInfo.VILLAGER_TEAM.value)
    FREEMASON = Role(en="FREEMASON", ja="共有者", team=RoleTeamInfo.VILLAGER_TEAM.value)

    WEREWOLF = Role(en="WEREWOLF", ja="人狼", team=RoleTeamInfo.WEREWOLF_TEAM.value)
    POSSESSED = Role(en="POSSESSED", ja="狂人", team=RoleTeamInfo.WEREWOLF_TEAM.value)

    FOX = Role(en="FOX", ja="妖狐", team=RoleTeamInfo.FOX_TEAM.value)

    ANY = Role(en="ANY", ja="?", team=RoleTeamInfo.ANY_TEAM.value)

    @classmethod
    def is_exist_role(cls, role: str | Role) -> bool:
        is_exist = False
        for role_info in cls.__members__.values():
            if (
                role in {role_info, role_info.value}
                or role in role_info.value.get_translations()
            ):
                is_exist = True
        return is_exist

    @classmethod
    def is_villager(cls, role: str | Role) -> bool:
        return (
            role in {cls.VILLAGER, cls.VILLAGER.value}
            or role in cls.VILLAGER.value.get_translations()
        )

    @classmethod
    def is_seer(cls, role: str | Role) -> bool:
        return (
            role in {cls.SEER, cls.SEER.value}
            or role in cls.SEER.value.get_translations()
        )

    @classmethod
    def is_medium(cls, role: str | Role) -> bool:
        return (
            role in {cls.MEDIUM, cls.MEDIUM.value}
            or role in cls.MEDIUM.value.get_translations()
        )

    @classmethod
    def is_bodyguard(cls, role: str | Role) -> bool:
        return (
            role in {cls.BODYGUARD, cls.BODYGUARD.value}
            or role in cls.BODYGUARD.value.get_translations()
        )

    @classmethod
    def is_werewolf(cls, role: str | Role) -> bool:
        return (
            role in {cls.WEREWOLF, cls.WEREWOLF.value}
            or role in cls.WEREWOLF.value.get_translations()
        )

    @classmethod
    def is_possessed(cls, role: str | Role) -> bool:
        return (
            role in {cls.POSSESSED, cls.POSSESSED.value}
            or role in cls.POSSESSED.value.get_translations()
        )

    @classmethod
    def is_villager_team(cls, role: str) -> bool:
        if not cls.is_exist_role(role=role):
            raise ValueError(role + "is not exist role.")
        for role_info in cls.__members__.values():
            if (
                role in {role_info, role_info.value}
                or role in role_info.value.get_translations()
            ):
                return role_info.value.team == RoleTeamInfo.VILLAGER_TEAM.value
        return False

    @classmethod
    def is_werewolf_team(cls, role: str) -> bool:
        if not cls.is_exist_role(role=role):
            raise ValueError(role + "is not exist role.")
        for role_info in cls.__members__.values():
            if (
                role in {role_info, role_info.value}
                or role in role_info.value.get_translations()
            ):
                return role_info.value.team == RoleTeamInfo.WEREWOLF_TEAM.value
        return False

    @classmethod
    def get_role_info(cls, role: str) -> Role:
        for role_info in cls.__members__.values():
            if role == role_info or role in role_info.value.get_translations():
                return role_info.value
        raise ValueError(role + "is not exist role.")



from __future__ import annotations

import json
import re


class Client:
    encode = "utf-8"

    def __init__(self) -> None:
        pass

    def receive(self) -> list[str] | str | None:
        pass

    def send(self, req: str) -> None:
        pass

    def close(self) -> None:
        pass

    @classmethod
    def is_valid(cls, resp: bytes) -> bool:
        try:
            resp_str = resp.decode(cls.encode)
        except UnicodeDecodeError:
            return False
        if resp_str == "":
            return False
        try:
            json.loads(resp_str)
        except ValueError:
            return False
        return True

    @classmethod
    def split_receives(cls, receive: str, *, include_newline: bool = True) -> list[str]:
        if include_newline:
            return re.findall("({.*})\n", receive)
        return re.findall("({.*})", receive)
